import { execSync } from 'child_process'
import { readdirSync } from 'fs'
import { join } from 'path'
import { config } from '../config'
import ora from 'ora'

export function deploy() {
  const spinner = ora('Deploying functions').start()

  const functions = readdirSync(config.functionsDir).filter((f) =>
    f.endsWith('.ts')
  )

  for (const functionFile of functions) {
    const functionName = functionFile.replace('.ts', '')
    spinner.text = `Deploying ${functionName}`

    config.env.forEach((e) => {
      if (!process.env[e]) {
        throw new Error(`Missing environment variable ${e}`)
      }
    })

    execSync(
      `gcloud functions deploy ${functionName} \
      --runtime ${config.runtime} \
      --trigger-http \
      --region=${config.region} \
      --allow-unauthenticated \
      --source=${join(config.buildDir, functionName)} \
      --entry-point=${functionName} \
      ${config.env
        .map((e: string) => `--set-env-vars="${e}=$${process.env[e]}"`)
        .join(' ')}`
    )

    spinner.succeed(`Deployed ${functionName}`)
  }
}
